@@ -1,17 +1,18 @@
 import logging
 import random
 import time
+from plistlib import Dict
 
-from frenetic.executors.abstract_executor import AbstractExecutor
-from frenetic.utils.random import seeded_rng
+from freneticlib.executors.executor import Executor
+from freneticlib.utils.random import seeded_rng
 
 logger = logging.getLogger(__name__)
 
 
-class MockExecutor(AbstractExecutor):
+class MockExecutor(Executor):
     """Inspired by SBST pipeline"""
 
-    def _execute(self, the_test) -> dict:
+    def _execute(self, the_test) -> Dict:
         test_outcome = seeded_rng().choice(["FAIL"] * 3 + ["PASS"] * 5 + ["ERROR"])
         description = "Mocked test results"
 
